#include <math.h>
#include <CheapStepper.h>
#include <HCMotor.h>
#include <pitches.h>
#include <SimpleSDAudio.h>

#define DISTANCE 0.075  //DISTANCE BETWEEN SENSORS
#define PAUSE 500
#define STEPS 4096

int  Led13 = 13;

bool sound_detected = false;

//GLOBAL VARIABLES USED FOR CALCULATING AND COUNTING
float t1;  //time 1
float t2;  //time2
float t3;  //time33
float rad;  //sound location in radians
float deg;  //sound location in degrees

//STEPPER MOTOR
CheapStepper stepper;
bool moveClockwise = true;  //FLAG TO MOVE CLOCKWISE OR COUNTERCLOCKWISE
int endposition = STEPS/4;
int nullposition = 0;
int targetposition = 0;

//WATER PUMP
#define MOTOR_PIN 3
int Speed = 200;
HCMotor HCMotor;

//MELODY
int melody[] = {
  NOTE_G3, NOTE_D4, NOTE_G3, END
};
int note_durations[] = {
  8, 8, 16, END
};
int playback_speed = 90;

void setup() {
  // INITIALIZE WATER PUMP
  HCMotor.Init();
  HCMotor.attach(0, DCMOTOR, MOTOR_PIN);
  HCMotor.DutyCycle(0, 100);
  
  //DEFINE OUT AND IN - DDRD
  DDRD = B01101111;  //SETS PINS 7 AND 4 AS INPUTS FOR MICROPHONES
  
  Serial.begin(9600);

  pinMode(Led13,OUTPUT);  // DEFINE LED13 AS OUTPUT TO SHOW THAT TRIGGER WAS REACHED
  
  initialize_stepper();
  delay(1000);

  // Initialize SimpleSDAudio
  SdPlay.setSDCSPin(4); // sd card cs pin
  if (!SdPlay.init(SSDA_MODE_FULLRATE | SSDA_MODE_MONO | SSDA_MODE_AUTOWORKER)) {
    while(1);
  }
  if (!SdPlay.setFile("music.wav")) { // music file name
    while(1);
  }
}

void loop() {
  search();  //MOVE RANDOMLY TO SEARCH FOR SOUND SOURCE
  //detect_sound_direction();  //TRY TO DETECT SOUND AND ITS DIRECTION
  detect_sound_direction_demo(); // FOR DEMONSTRATION ONLY
  
  if (sound_detected == true) {
    move_angle(deg);  //MOVE TO SOUNDSOURCE
    warn();  //PLAY SOUND
    shoot();  //SHOOT
  }

  sound_detected = false;
  delay(500);
}

// PLAY SOUND TO WARN TARGET
void warn() {
  Serial.println("ATTENTION: Play Sound!");
}

// SHOOT WITH WATERGUN
void shoot()  {
  HCMotor.OnTime(0, Speed);
  Serial.println("SHOOT!");
  delay(1000);
  HCMotor.OnTime(0, 0);
  delay(200);
  
  // Play sound after shooting
  SdPlay.play(); // play music
  while (!SdPlay.isStopped()) {
    // wait for the music to finish
  }
  Serial.println("Sound played!");
}

// MOVE RANDOMLY FROM LEFT TO RIGHT
void search() {
  int randint = random(-9000, 9000);
  float randflt = (float)randint / 100.00;
  move_angle(randflt);
  delay(500);
  Serial.println("Done searching!");
}

// MOVE STEPPER MOTOR BY ANGLE
void move_angle(float angle)  {
  int steps = angle2steps(angle);
  move_steps(steps);
}

// MOVE STEPPER MOTOR BY STEPS
void move_steps(int steps) {
  bool clockwise = true;
  if (steps < 0.0)  {
    clockwise = false;
  }
  steps = abs(steps);
  for (int s=0; s<steps; s++) {
    stepper.step(clockwise);
    int nStep = stepper.getStep(); 
    if (nStep%64==0)  {
    Serial.print("current step position: "); Serial.print(nStep);
    Serial.println();
    }
  }
}

// DETECT DIRECTION OF SOUND AS OFFSET ANGLE
void detect_sound_direction()  {
  //FLUSH VARIABLES
  t1 = 0;
  t2 = 0;
  t3 = 0;
  rad = 0;
  deg = 0;

  //TURN OFF LED (L)
  digitalWrite(Led13,LOW);

  /*
  Code uses direct port manipulation!
  PIND reads value of pins 0-7 at the same time,
  and returns them as byte, the digit position 
  equals the pin number, i.e. B00000001 means Pin0 HIGH
  */

  //IF PIN 7 AND PIND, NOTE TIME AND CALULATE ANGLE
  if((PIND & B10000000) != 0) {
    t1 = micros();
    digitalWrite(Led13,HIGH);
    //WAIT FOR PIN 4
    while((PIND & B00010000)==0)  {
    }
    //NOTE TIME PIN 3
    t2 = micros();
    //CALCULATE TIME DIFFERENCE, LEFT SIDE IS NEGATIVE
    t3 = -1 * (t2 - t1);
    //IF TIME DIFFERENCE EXCEEDS 1sec, SKIP
    if(abs(t3) > 400) {
    }
    else  {
      //CALCULATE ANGLE AND CONVERT FROM RAD TO DEG
      rad = calc_rad(t3);
      deg = rad2deg(rad);
      sound_detected = true;
      Serial.print("Left Sensor: "); Serial.print(deg);
      Serial.println();
    }
    delay(PAUSE);
  }
  //IF PIN 4 AND PIND, NOTE TIME AND CALULATE ANGLE
  if((PIND & B00010000) != 0) {
    t1 = micros();
    digitalWrite(Led13,HIGH);
    //WAIT FOR PIN 7
    while((PIND & B10000000) == 0)  {
    }
    //NOTE TIME PIN 5
    t2 = micros();
    //CALCULATE TIME DIFFERENCE, RIGHT SIDE IS POSITIVE
    t3 = t2 - t1;
    //IF TIME EXCEEDS 1sec, SKIP
    if(abs(t3) > 400) {
    }
    else  {
      //CALCULATE ANGLE AND CONVERT FROm RAD TO DEG
      rad = calc_rad(t3);
      deg = rad2deg(rad);
      sound_detected = true;
      Serial.print("Right Sensor: "); Serial.print(deg);
      Serial.println();
    }
    delay(PAUSE);
  }
}

//INITIALIZE STEPPER MOTOR
void initialize_stepper() {
  int pos = stepper.getStep();
  if (pos > nullposition) {
    move_steps(nullposition - pos);
  }
  else if (pos < nullposition)  {
    move_steps(nullposition - pos);
  }
  else  {
  }

  delay(100);

  check_stepper();
  Serial.println("stepper motor ready");
}

//CHECK IF STEPPER CAN REACH END POSITIONS
void check_stepper() {
  move_steps(endposition);
  move_steps(-endposition);
  delay(1000);
  move_steps(-endposition);
  move_steps(endposition);
}

// PLAY SOUND (CONFLICTING WITH HC MOTOR LIBRARY)
// void play_sound() {
//   Serial.println("Playing melody...");
//   for (int thisNote = 0; melody[thisNote]!=-1; thisNote++) {
//     int noteDuration = playback_speed*note_durations[thisNote];
//     tone(6, melody[thisNote],noteDuration*.95);
//     Serial.println(melody[thisNote]);
//     delay(noteDuration);
//     noTone(6);
//   }
// }

//CALCULATE WHERE AUDIO IS COMING FROM
float calc_rad (float time) {
  return (asin((time*0.000001*343.00)/(DISTANCE)));  //RETURN ANS. IN RADIANS
}

//CONVERTS FROM RADIANS TO DEGREES
float rad2deg (float rad) {
  return ((rad/(2*PI))*360);
}

// CONVERT ANGLE TO MOTOR STEPS
int angle2steps(float angle)  {
  float offset_steps = angle * STEPS / 360.0;
  int steps = static_cast<int>(offset_steps);
  return steps;
}

// DEMO SOUND DETECTION
void detect_sound_direction_demo() {
  int randint = random (-3000, 3000);
  float randangle = static_cast<float>(randint) / 100.0;

  if (randint > 2000 || randint < -2000){
    deg = randangle;
    sound_detected = true;
    Serial.print("Demo Sensor: "); Serial.print(deg);
    Serial.println();
  }
}
